package com.tzld.piaoquan.tencentad.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.base.ExceptionCodeEnum;
import com.tzld.piaoquan.tencentad.common.constants.TimeConstant;
import com.tzld.piaoquan.tencentad.common.enums.AdMuseTaskStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoStatusEnum;
import com.tzld.piaoquan.tencentad.common.enums.SimilarityCodeEnum;
import com.tzld.piaoquan.tencentad.component.HttpPoolClient;
import com.tzld.piaoquan.tencentad.dao.mapper.AdGroupMapper;
import com.tzld.piaoquan.tencentad.dao.mapper.AdMuseTaskMapper;
import com.tzld.piaoquan.tencentad.dao.mapper.AdVideoMapper;
import com.tzld.piaoquan.tencentad.model.bo.ImageParam;
import com.tzld.piaoquan.tencentad.model.po.*;
import com.tzld.piaoquan.tencentad.model.vo.AdMuseImageParam;
import com.tzld.piaoquan.tencentad.model.vo.AdMuseImageVo;
import com.tzld.piaoquan.tencentad.service.AdAccountService;
import com.tzld.piaoquan.tencentad.service.AdMuseTaskService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.ImageUtils;
import com.tzld.piaoquan.tencentad.utils.OSSUploader;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;

@Slf4j
@Service
public class AdMuseTaskServiceImpl implements AdMuseTaskService {

    @Value("${image_path}")
    private String imagePath;

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AdMuseTaskMapper adMuseTaskMapper;

    @Autowired
    private AdVideoMapper adVideoMapper;

    @Autowired
    private AdGroupMapper adGroupMapper;

    @Autowired
    private AdAccountService accountService;

    public void addMuseTask(String accountId, String accessToken, AdVideoImage adVideoImage) {
        try {
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_task/add?access_token=%s&timestamp=%s&nonce=%s",
                    accessToken, System.currentTimeMillis() / 1000, UUID.randomUUID().toString().replaceAll("-", ""));
            JSONObject param = new JSONObject();
            param.put("account_id", accountId);
            if (StringUtils.isNotEmpty(adVideoImage.getText2imgDesc())) {
                param.put("task_type", "MUSE_AI_TASK_TYPE_TEXT2IMG");
                JSONObject text2img = new JSONObject();
                if (StringUtils.isEmpty(adVideoImage.getDimensionSize())) {
                    text2img.put("dimension_size", adVideoImage.getDimensionSize());
                } else {
                    text2img.put("dimension_size", "SIZE_1280_720");
                }
                text2img.put("prompt", adVideoImage.getText2imgDesc());
                param.put("text2img", text2img);
            } else {
                param.put("task_type", "MUSE_AI_TASK_TYPE_IMG2IMG");
                JSONObject img2img = new JSONObject();
                //默认取4级相似
                img2img.put("image_id", adVideoImage.getTencentImageId());
                img2img.put("similarity", adVideoImage.getSimilarity() == null ?
                        SimilarityCodeEnum.FOURTH.getCode() : adVideoImage.getSimilarity());
                img2img.put("img2img_style", "MUSE_AI_IMG2IMG_STYLE_PEOPLE");
                if (StringUtils.isNotEmpty(adVideoImage.getDimensionSize())) {
                    img2img.put("dimension_size", adVideoImage.getDimensionSize());
                } else {
                    img2img.put("extend_size", true);
                }

                param.put("img2img", img2img);
            }
            String res = httpPoolClient.post(url, param.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                AdMuseTask adMuseTask = new AdMuseTask();
                adMuseTask.setTaskId(data.getLong("task_id"));
                adMuseTask.setTaskStatus(data.getString("task_status"));
                adMuseTask.setTaskType(data.getString("task_type"));
                adMuseTask.setAdVideoId(adVideoImage.getAdVideoId());
                adMuseTask.setAdVideoImageId(adVideoImage.getId());
                adMuseTaskMapper.insertSelective(adMuseTask);
            }
        } catch (Exception e) {
            log.error("createMuseTask error ", e);
        }
    }

    public List<AdMuseTask> getAdMuseTask(Long adVideoId, Integer status) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andStatusEqualTo(status);
        return adMuseTaskMapper.selectByExample(example);
    }

    public List<AdMuseTask> getAdMuseTask(Long adVideoId, Integer status, List<Long> adMuseTaskIds) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId).andStatusEqualTo(status).andIdIn(adMuseTaskIds);
        return adMuseTaskMapper.selectByExample(example);
    }

    public boolean getMuseTaskRes(String accountId, String accessToken, AdMuseTask adMuseTask, String caption) {
        try {
            long now = System.currentTimeMillis();
            long create = adMuseTask.getCreateTime().getTime();
            if (now < create + 3 * TimeConstant.MILLISECOND_MINUTE) {
                return false;
            }
            long timestamp = System.currentTimeMillis() / 1000;
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_task/get?access_token=%s&timestamp=%s&nonce=%s" +
                            "&account_id=%s&task_id=%s",
                    accessToken, timestamp, timestamp, accountId, adMuseTask.getTaskId());
            String res = httpPoolClient.get(url);
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                String taskStatus = data.getString("task_status");
                if ("MUSE_AI_TASK_SUCCESS".equals(taskStatus)) {
                    adMuseTask.setTaskStatus(taskStatus);
                    JSONObject result = data.getJSONArray("results").getJSONObject(0);
                    adMuseTask.setMuseMaterialId(result.getLong("muse_material_id"));
                    adMuseTask.setMuseUrl(result.getString("url"));
                    adMuseTask.setStitchUrl(getStitchUrl(result.getString("url"), caption));
                    adMuseTask.setStatus(AdMuseTaskStatusEnum.CREATE_SUCCESS.getStatus());
                    adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
                    return true;
                }
                if ("MUSE_AI_TASK_FAIL".equals(taskStatus)) {
                    adMuseTask.setTaskStatus(taskStatus);
                    adMuseTask.setStatus(AdMuseTaskStatusEnum.CREATE_FAIL.getStatus());
                    adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
                    return true;
                }
            }
        } catch (Exception e) {
            log.error("createMuseTask error ", e);
        }
        return false;
    }

    public String getStitchUrl(String url, String caption) {
        if (StringUtils.isEmpty(url)) {
            return null;
        }
        String targetImgPath = imagePath + "/" + UUID.randomUUID() + ".jpg";
        ImageParam imageParam = new ImageParam();
        imageParam.setFont(new Font("微软雅黑", Font.PLAIN, 48));
        imageParam.setColor(new Color(102, 102, 102));
        imageParam.setText(caption);
        return ImageUtils.writeImageFromUrl(url, targetImgPath, imageParam);
    }

    public long selectCount(Long adVideoId, List<Integer> status) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        AdMuseTaskExample.Criteria criteria = example.createCriteria().andAdVideoIdEqualTo(adVideoId);
        if (!CollectionUtils.isEmpty(status)) {
            criteria.andStatusIn(status);
        }
        return adMuseTaskMapper.countByExample(example);
    }

    @Override
    public CommonResponse<List<AdMuseImageVo>> getAdMuseImageVoList(Long adVideoId) {
        AdMuseTaskExample example = new AdMuseTaskExample();
        example.createCriteria().andAdVideoIdEqualTo(adVideoId);
        List<AdMuseTask> adMuseTasks = adMuseTaskMapper.selectByExample(example);
        List<AdMuseImageVo> res = new ArrayList<>();
        if (CollectionUtils.isEmpty(adMuseTasks)) {
            return CommonResponse.success(res);
        }
        for (AdMuseTask adMuseTask : adMuseTasks) {
            AdMuseImageVo adMuseImageVo = new AdMuseImageVo();
            BeanUtils.copyProperties(adMuseTask, adMuseImageVo);
            AdMuseTaskStatusEnum adMuseTaskStatusEnum = AdMuseTaskStatusEnum.getByStatus(adMuseTask.getStatus());
            if (adMuseTaskStatusEnum != null) {
                adMuseImageVo.setStatusName(adMuseTaskStatusEnum.getDesc());
            }
            adMuseImageVo.setCreateTime(DateUtils.getDateString(adMuseTask.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adMuseImageVo.setUpdateTime(DateUtils.getDateString(adMuseTask.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            res.add(adMuseImageVo);
        }
        return CommonResponse.success(res);
    }


    public CommonResponse<Void> updateMuseTask(AdMuseImageParam adMuseImageParam) {
        if (adMuseImageParam == null || adMuseImageParam.getId() == null || adMuseImageParam.getStatus() == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误");
        }
        List<Integer> allowStatus = Arrays.asList(AdMuseTaskStatusEnum.VERIFY_SUCCESS.getStatus(), AdMuseTaskStatusEnum.VERIFY_FAIL.getStatus());
        if (!allowStatus.contains(adMuseImageParam.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，状态无效");
        }
        AdMuseTask adMuseTask = adMuseTaskMapper.selectByPrimaryKey(adMuseImageParam.getId());
        if (adMuseTask == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，数据不存在");
        }
        if (!Objects.equals(adMuseTask.getStatus(), AdMuseTaskStatusEnum.CREATE_SUCCESS.getStatus())) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "该状态不允许审核");
        }
        adMuseTask.setStatus(adMuseImageParam.getStatus());
        adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
        if (AdMuseTaskStatusEnum.VERIFY_SUCCESS.getStatus().equals(adMuseImageParam.getStatus())) {
            AdVideo adVideo = adVideoMapper.selectByPrimaryKey(adMuseTask.getAdVideoId());
            String accessToken = accountService.getAccessToken(adVideo.getAccountId());
            String url = StringUtils.isEmpty(adMuseTask.getFinalUrl()) ? adMuseTask.getStitchUrl() : adMuseTask.getFinalUrl();
            Long mediaId = uploadImage(adVideo.getAccountId(), accessToken, url, adMuseTask.getMuseMaterialId());
            if (mediaId != null) {
                adMuseTask.setMediaId(mediaId);
                adMuseTask.setStatus(AdMuseTaskStatusEnum.UPLOAD_SUCCESS.getStatus());
                adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
                AdVideo updateAdVideo = new AdVideo();
                updateAdVideo.setId(adVideo.getId());
                updateAdVideo.setStatus(AdVideoStatusEnum.MUSE_UPLOAD_SUCCESS.getStatus());
                adVideoMapper.updateByPrimaryKeySelective(updateAdVideo);
            } else {
                adMuseTask.setStatus(AdMuseTaskStatusEnum.UPLOAD_FAIL.getStatus());
                adMuseTaskMapper.updateByPrimaryKeySelective(adMuseTask);
            }
        }
        return CommonResponse.success();
    }

    @Override
    public CommonResponse<Void> uploadMuseImage(MultipartFile file, Long id) {
        if (id == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，id不能为空");
        }
        // 创建上传目录
        String fileName = file.getOriginalFilename();
        if (fileName == null) {
            return CommonResponse.create(ExceptionCodeEnum.PARAM_ERROR.getCode(), "参数错误，文件名无效");
        }
        File uploadDir = new File(imagePath);
        if (!uploadDir.exists()) {
            uploadDir.mkdirs(); // 创建目录
        }
        String path = imagePath + "/" + fileName;
        // 保存文件
        File destinationFile = new File(uploadDir, fileName);
        try {
            file.transferTo(destinationFile);
        } catch (IOException e) {
            return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
        }
        String url = OSSUploader.uploadToOSS(path);
        if (StringUtils.isNotEmpty(url)) {
            AdMuseTask updateAdMuseTask = new AdMuseTask();
            updateAdMuseTask.setId(id);
            updateAdMuseTask.setFinalUrl(url);
            adMuseTaskMapper.updateByPrimaryKeySelective(updateAdMuseTask);
            return CommonResponse.success();
        }
        return CommonResponse.create(ExceptionCodeEnum.SYS_ERROR.getCode(), "系统异常，上传失败");
    }

    public Long uploadImage(String accountId, String accessToken, String imageUrl, Long museMaterialId) {
        try {
            long timestamp = System.currentTimeMillis() / 1000;
            String url = String.format("https://api.e.qq.com/v3.0/muse_ai_ugc/add?access_token=%s&timestamp=%s&nonce=%s",
                    accessToken, timestamp, timestamp);
            JSONObject param = new JSONObject();
            param.put("account_id", accountId);
            param.put("muse_material_id", museMaterialId);
            param.put("media_base64_str", ImageUtils.imageUrlToBase64(imageUrl));
            String res = httpPoolClient.post(url, param.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                return data.getLong("media_id");
            }
        } catch (Exception e) {
            log.error("uploadImage error", e);
        }
        return null;
    }


}
